function [fracs,Ebars,idxs,idx_threshold] = BinFracs(sp,Ethresholds)
% function [fracs,Ebars,idxs,idx_threshold] = BinFracs(sp,Ethresholds)
%  Compute fractions of photons in energy bins specified by threshold energies Ethresholds
%  inputs:
%      sp: a spectrum struct with fields
%          egys: the photon energies (keV)--assumed equally spaced
%          specnum: the number spectrum
%       Ethresholds: a vector with threshold energies assumed to be monotonically increasing
%  outputs:
%      fracs: the fractions. Sum(fracs) = 1
%      Ebars: the average energy of each bin
%      idxs: a cell array of the indexes into sp.egys array for the energy bins
%      idx_threshold: the indexes into sp.egys array corresponding to Ethresholds
%  REA 2011-Jul-24 10:03
%  COPYRIGHT:   Aprend Technology, 2011. All rights reserved.

nthresh = numel(Ethresholds);
idx_threshold = zeros(nthresh,1);
for k = 1:nthresh
  [minval,idx] = min(abs(Ethresholds(k) - sp.egys));
  idx_threshold(k) = idx;
end

nbins = nthresh + 1;

bin_edges = [0; idx_threshold(:);numel(sp.egys)];
idxs = cell(nbins,1);
specnum = sp.specnum/sum(sp.specnum); % normalize it
specnum = specnum(:); % column vector
fracs = zeros(nbins,1);
Ebars = zeros(nbins,1);
sp.egys = sp.egys(:); % column vector
for k = 1:nbins
    idxs{k}= (bin_edges(k)+1):bin_edges(k+1);
    fracs(k) = sum(specnum(idxs{k}));
    Ebars(k) = sum(sp.egys(idxs{k}).*specnum(idxs{k}))/fracs(k);
end

