function [counts,egys_rec,idxs_start,photon_energies] = PileupPulses(dt_pulse,deadtime,t_integrate,varargin)
%  inputs: 
%    dt_pulse: the input inter-pulse intervals. Note all times must be in same units
%    deadtime: the dead time
%    t_integrate: the total time to count photons
%    specinv: (optional = []) spectrum cumulative inverse if present, generates energies of recorded pulses
%  outputs:
%   counts = [npulses kpulse];
%            npulses: the number of recorded pulses
%            kpulse: the number of input pulses during the integration time
%   egys_rec: recorded energies of counted pulses
%   idxs_start: the indexes into the dt_pulse vector for the start of the deadtimes
%   photon_energies: the energies of each photon for each value of dt_pulse

nreqargs = 3;
if(nargin<nreqargs), error('%d args required, only have %d',nreqargs,nargin);end
specinv = [];
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'specinv';         specinv=varargin{i+1};   
         assert(isstruct(specinv) && isfield(specinv,'egy_min') && isfield(specinv,'specuminv') && ...
            isfield(specinv,'ys') );  
         i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end
  % loop gets hung up if deadtime==0 so set it to small number for 0 case
if deadtime < eps
    deadtime = eps;
end
n = numel(dt_pulse);
egys_rec = [];
photon_energies = [];
if isempty(specinv)
    do_egys = false;
else
     do_egys = true;
     egys_rec = zeros(n,1);
%      rng(4)
     photon_energies = interp1(specinv.ys,specinv.specuminv,rand(numel(dt_pulse),1));
end

npulses = 0; % counted pulses
kpulse = 1; % counter into dt_pulses vector
tproc = dt_pulse(1);
idxs_start = zeros(n,1);
while tproc < t_integrate
    npulses = npulses + 1;
    idxs_start(npulses) = kpulse;
    if do_egys, egy = 0; end
    tend = tproc + deadtime;    
    while (kpulse<= n) && (tproc < tend)
        if do_egys, egy = egy + photon_energies(kpulse); end
        kpulse = kpulse + 1;
        tproc = tproc + dt_pulse(kpulse); 
    end
    if do_egys, egys_rec(npulses) = egy; end
end
counts = [npulses kpulse];
idxs_start = idxs_start(1:npulses);
if do_egys, egys_rec = egys_rec(1:npulses); end
